@@ -22,33 +22,30 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/fs"
 	"os"
 	"path/filepath"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 
+	"github.com/compose-spec/compose-go/v2/paths"
 	"github.com/compose-spec/compose-go/v2/types"
-	"github.com/docker/compose/v2/internal/desktop"
-	pathutil "github.com/docker/compose/v2/internal/paths"
-	"github.com/docker/compose/v2/pkg/api"
-	"github.com/docker/compose/v2/pkg/progress"
-	"github.com/docker/compose/v2/pkg/prompt"
-	"github.com/docker/compose/v2/pkg/utils"
+	cerrdefs "github.com/containerd/errdefs"
 	"github.com/docker/docker/api/types/blkiodev"
 	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/api/types/filters"
-	"github.com/docker/docker/api/types/image"
 	"github.com/docker/docker/api/types/mount"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/strslice"
 	"github.com/docker/docker/api/types/versions"
 	volumetypes "github.com/docker/docker/api/types/volume"
-	"github.com/docker/docker/errdefs"
 	"github.com/docker/go-connections/nat"
 	"github.com/sirupsen/logrus"
 	cdi "tags.cncf.io/container-device-interface/pkg/parser"
+
+	"github.com/docker/compose/v2/pkg/api"
+	"github.com/docker/compose/v2/pkg/progress"
+	"github.com/docker/compose/v2/pkg/prompt"
 )
 
 type createOptions struct {
@@ -86,6 +83,11 @@ func (s *composeService) create(ctx context.Context, project *types.Project, opt
 		return err
 	}
 
+	err = s.ensureModels(ctx, project, options.QuietPull)
+	if err != nil {
+		return err
+	}
+
 	prepareNetworks(project)
 
 	networks, err := s.ensureNetworks(ctx, project)
@@ -117,6 +119,13 @@ func (s *composeService) create(ctx context.Context, project *types.Project, opt
 				"--remove-orphans flag to clean it up.", orphans.names())
 		}
 	}
+
+	// Temporary implementation of use_api_socket until we get actual support inside docker engine
+	project, err = s.useAPISocket(project)
+	if err != nil {
+		return err
+	}
+
 	return newConvergence(options.Services, observedState, networks, volumes, s).apply(ctx, project, options)
 }
 
@@ -156,61 +165,10 @@ func (s *composeService) ensureProjectVolumes(ctx context.Context, project *type
 		ids[k] = id
 	}
 
-	err := func() error {
-		if s.manageDesktopFileSharesEnabled(ctx) {
-			// collect all the bind mount paths and try to set up file shares in
-			// Docker Desktop for them
-			var paths []string
-			for _, svcName := range project.ServiceNames() {
-				svc := project.Services[svcName]
-				for _, vol := range svc.Volumes {
-					if vol.Type != string(mount.TypeBind) {
-						continue
-					}
-					p := filepath.Clean(vol.Source)
-					if !filepath.IsAbs(p) {
-						return fmt.Errorf("file share path is not absolute: %s", p)
-					}
-					if fi, err := os.Stat(p); errors.Is(err, fs.ErrNotExist) {
-						// actual directory will be implicitly created when the
-						// file share is initialized if it doesn't exist, so
-						// need to filter out any that should not be auto-created
-						if vol.Bind != nil && !vol.Bind.CreateHostPath {
-							logrus.Debugf("Skipping creating file share for %q: does not exist and `create_host_path` is false", p)
-							continue
-						}
-					} else if err != nil {
-						// if we can't read the path, we won't be able to make
-						// a file share for it
-						logrus.Debugf("Skipping creating file share for %q: %v", p, err)
-						continue
-					} else if !fi.IsDir() {
-						// ignore files & special types (e.g. Unix sockets)
-						logrus.Debugf("Skipping creating file share for %q: not a directory", p)
-						continue
-					}
-
-					paths = append(paths, p)
-				}
-			}
-
-			// remove duplicate/unnecessary child paths and sort them for predictability
-			paths = pathutil.EncompassingPaths(paths)
-			sort.Strings(paths)
-
-			fileShareManager := desktop.NewFileShareManager(s.desktopCli, project.Name, paths)
-			if err := fileShareManager.EnsureExists(ctx); err != nil {
-				return fmt.Errorf("initializing file shares: %w", err)
-			}
-		}
-		return nil
-	}()
-	if err != nil {
-		progress.ContextWriter(ctx).TailMsgf("Failed to prepare Synchronized file shares: %v", err)
-	}
 	return ids, nil
 }
 
+//nolint:gocyclo
 func (s *composeService) getCreateConfigs(ctx context.Context,
 	p *types.Project,
 	service types.ServiceConfig,
@@ -303,7 +261,10 @@ func (s *composeService) getCreateConfigs(ctx context.Context,
 	if err != nil {
 		return createConfigs{}, err
 	}
-	networkMode, networkingConfig := defaultNetworkSettings(p, service, number, links, opts.UseNetworkAliases, apiVersion)
+	networkMode, networkingConfig, err := defaultNetworkSettings(p, service, number, links, opts.UseNetworkAliases, apiVersion)
+	if err != nil {
+		return createConfigs{}, err
+	}
 	portBindings := buildContainerPortBindingOptions(service)
 
 	// MISC
@@ -413,7 +374,7 @@ func (s *composeService) prepareContainerMACAddress(ctx context.Context, service
 		}
 
 		if len(withMacAddress) > 1 {
-			return "", fmt.Errorf("a MAC address is specified for multiple networks (%s), but this feature requires Docker Engine 1.44 or later (currently: %s)", strings.Join(withMacAddress, ", "), version)
+			return "", fmt.Errorf("a MAC address is specified for multiple networks (%s), but this feature requires Docker Engine v25 or later", strings.Join(withMacAddress, ", "))
 		}
 
 		if mainNw != nil && mainNw.MacAddress != "" {
@@ -436,6 +397,8 @@ func getAliases(project *types.Project, service types.ServiceConfig, serviceInde
 }
 
 func createEndpointSettings(p *types.Project, service types.ServiceConfig, serviceIndex int, networkKey string, links []string, useNetworkAliases bool) *network.EndpointSettings {
+	const ifname = "com.docker.network.endpoint.ifname"
+
 	config := service.Networks[networkKey]
 	var ipam *network.EndpointIPAMConfig
 	var (
@@ -455,6 +418,15 @@ func createEndpointSettings(p *types.Project, service types.ServiceConfig, servi
 		}
 		macAddress = config.MacAddress
 		driverOpts = config.DriverOpts
+		if config.InterfaceName != "" {
+			if driverOpts == nil {
+				driverOpts = map[string]string{}
+			}
+			if name, ok := driverOpts[ifname]; ok && name != config.InterfaceName {
+				logrus.Warnf("ignoring services.%s.networks.%s.interface_name as %s driver_opts is already declared", service.Name, networkKey, ifname)
+			}
+			driverOpts[ifname] = config.InterfaceName
+		}
 		gwPriority = config.GatewayPriority
 	}
 	return &network.EndpointSettings{
@@ -486,7 +458,7 @@ func parseSecurityOpts(p *types.Project, securityOpts []string) ([]string, bool,
 			if strings.Contains(opt, ":") {
 				con = strings.SplitN(opt, ":", 2)
 			} else {
-				return securityOpts, false, fmt.Errorf("Invalid security-opt: %q", opt)
+				return securityOpts, false, fmt.Errorf("invalid security-opt: %q", opt)
 			}
 		}
 		if con[0] == "seccomp" && con[1] != "unconfined" && con[1] != "builtin" {
@@ -528,20 +500,17 @@ func (s *composeService) prepareLabels(labels types.Labels, service types.Servic
 }
 
 // defaultNetworkSettings determines the container.NetworkMode and corresponding network.NetworkingConfig (nil if not applicable).
-func defaultNetworkSettings(
-	project *types.Project,
-	service types.ServiceConfig,
-	serviceIndex int,
-	links []string,
-	useNetworkAliases bool,
+func defaultNetworkSettings(project *types.Project,
+	service types.ServiceConfig, serviceIndex int,
+	links []string, useNetworkAliases bool,
 	version string,
-) (container.NetworkMode, *network.NetworkingConfig) {
+) (container.NetworkMode, *network.NetworkingConfig, error) {
 	if service.NetworkMode != "" {
-		return container.NetworkMode(service.NetworkMode), nil
+		return container.NetworkMode(service.NetworkMode), nil, nil
 	}
 
 	if len(project.Networks) == 0 {
-		return "none", nil
+		return "none", nil, nil
 	}
 
 	var primaryNetworkKey string
@@ -572,6 +541,14 @@ func defaultNetworkSettings(
 		}
 	}
 
+	if versions.LessThan(version, "1.49") {
+		for _, config := range service.Networks {
+			if config != nil && config.InterfaceName != "" {
+				return "", nil, fmt.Errorf("interface_name requires Docker Engine v28.1 or later")
+			}
+		}
+	}
+
 	endpointsConfig[primaryNetworkMobyNetworkName] = primaryNetworkEndpoint
 	networkConfig := &network.NetworkingConfig{
 		EndpointsConfig: endpointsConfig,
@@ -580,7 +557,7 @@ func defaultNetworkSettings(
 	// From the Engine API docs:
 	// > Supported standard values are: bridge, host, none, and container:<name|id>.
 	// > Any other value is taken as a custom network's name to which this container should connect to.
-	return container.NetworkMode(primaryNetworkMobyNetworkName), networkConfig
+	return container.NetworkMode(primaryNetworkMobyNetworkName), networkConfig, nil
 }
 
 func getRestartPolicy(service types.ServiceConfig) container.RestartPolicy {
@@ -837,38 +814,48 @@ func (s *composeService) buildContainerVolumes(
 	var mounts []mount.Mount
 	var binds []string
 
-	img := api.GetImageNameOrDefault(service, p.Name)
-	imgInspect, err := s.apiClient().ImageInspect(ctx, img)
+	mountOptions, err := s.buildContainerMountOptions(ctx, p, service, inherit)
 	if err != nil {
 		return nil, nil, err
 	}
 
-	mountOptions, err := buildContainerMountOptions(p, service, imgInspect, inherit)
-	if err != nil {
-		return nil, nil, err
-	}
-
-MOUNTS:
 	for _, m := range mountOptions {
-		if m.Type == mount.TypeNamedPipe {
-			mounts = append(mounts, m)
-			continue
-		}
-		if m.Type == mount.TypeBind {
+		switch m.Type {
+		case mount.TypeBind:
 			// `Mount` is preferred but does not offer option to created host path if missing
 			// so `Bind` API is used here with raw volume string
 			// see https://github.com/moby/moby/issues/43483
-			for _, v := range service.Volumes {
-				if v.Target == m.Target {
-					switch {
-					case string(m.Type) != v.Type:
-						v.Source = m.Source
-						fallthrough
-					case !requireMountAPI(v.Bind):
-						binds = append(binds, v.String())
-						continue MOUNTS
-					}
+			v := findVolumeByTarget(service.Volumes, m.Target)
+			if v != nil {
+				if v.Type != types.VolumeTypeBind {
+					v.Source = m.Source
 				}
+				if !bindRequiresMountAPI(v.Bind) {
+					source := m.Source
+					if vol := findVolumeByName(p.Volumes, m.Source); vol != nil {
+						source = m.Source
+					}
+					binds = append(binds, toBindString(source, v))
+					continue
+				}
+			}
+		case mount.TypeVolume:
+			v := findVolumeByTarget(service.Volumes, m.Target)
+			vol := findVolumeByName(p.Volumes, m.Source)
+			if v != nil && vol != nil {
+				// Prefer the bind API if no advanced option is used, to preserve backward compatibility
+				if !volumeRequiresMountAPI(v.Volume) {
+					binds = append(binds, toBindString(vol.Name, v))
+					continue
+				}
+			}
+		case mount.TypeImage:
+			version, err := s.RuntimeVersion(ctx)
+			if err != nil {
+				return nil, nil, err
+			}
+			if versions.LessThan(version, "1.48") {
+				return nil, nil, fmt.Errorf("volume with type=image require Docker Engine v28 or later")
 			}
 		}
 		mounts = append(mounts, m)
@@ -876,9 +863,45 @@ MOUNTS:
 	return binds, mounts, nil
 }
 
-// requireMountAPI check if Bind declaration can be implemented by the plain old Bind API or uses any of the advanced
+func toBindString(name string, v *types.ServiceVolumeConfig) string {
+	access := "rw"
+	if v.ReadOnly {
+		access = "ro"
+	}
+	options := []string{access}
+	if v.Bind != nil && v.Bind.SELinux != "" {
+		options = append(options, v.Bind.SELinux)
+	}
+	if v.Bind != nil && v.Bind.Propagation != "" {
+		options = append(options, v.Bind.Propagation)
+	}
+	if v.Volume != nil && v.Volume.NoCopy {
+		options = append(options, "nocopy")
+	}
+	return fmt.Sprintf("%s:%s:%s", name, v.Target, strings.Join(options, ","))
+}
+
+func findVolumeByName(volumes types.Volumes, name string) *types.VolumeConfig {
+	for _, vol := range volumes {
+		if vol.Name == name {
+			return &vol
+		}
+	}
+	return nil
+}
+
+func findVolumeByTarget(volumes []types.ServiceVolumeConfig, target string) *types.ServiceVolumeConfig {
+	for _, v := range volumes {
+		if v.Target == target {
+			return &v
+		}
+	}
+	return nil
+}
+
+// bindRequiresMountAPI check if Bind declaration can be implemented by the plain old Bind API or uses any of the advanced
 // options which require use of Mount API
-func requireMountAPI(bind *types.ServiceVolumeBind) bool {
+func bindRequiresMountAPI(bind *types.ServiceVolumeBind) bool {
 	switch {
 	case bind == nil:
 		return false
@@ -893,7 +916,24 @@ func requireMountAPI(bind *types.ServiceVolumeBind) bool {
 	}
 }
 
-func buildContainerMountOptions(p types.Project, s types.ServiceConfig, img image.InspectResponse, inherit *container.Summary) ([]mount.Mount, error) {
+// volumeRequiresMountAPI check if Volume declaration can be implemented by the plain old Bind API or uses any of the advanced
+// options which require use of Mount API
+func volumeRequiresMountAPI(vol *types.ServiceVolumeVolume) bool {
+	switch {
+	case vol == nil:
+		return false
+	case len(vol.Labels) > 0:
+		return true
+	case vol.Subpath != "":
+		return true
+	case vol.NoCopy:
+		return true
+	default:
+		return false
+	}
+}
+
+func (s *composeService) buildContainerMountOptions(ctx context.Context, p types.Project, service types.ServiceConfig, inherit *container.Summary) ([]mount.Mount, error) {
 	mounts := map[string]mount.Mount{}
 	if inherit != nil {
 		for _, m := range inherit.Mounts {
@@ -905,6 +945,11 @@ func buildContainerMountOptions(p types.Project, s types.ServiceConfig, img imag
 				src = m.Name
 			}
 
+			img, err := s.apiClient().ImageInspect(ctx, api.GetImageNameOrDefault(service, p.Name))
+			if err != nil {
+				return nil, err
+			}
+
 			if img.Config != nil {
 				if _, ok := img.Config.Volumes[m.Destination]; ok {
 					// inherit previous container's anonymous volume
@@ -917,7 +962,7 @@ func buildContainerMountOptions(p types.Project, s types.ServiceConfig, img imag
 				}
 			}
 			volumes := []types.ServiceVolumeConfig{}
-			for _, v := range s.Volumes {
+			for _, v := range service.Volumes {
 				if v.Target != m.Destination || v.Source != "" {
 					volumes = append(volumes, v)
 					continue
@@ -930,11 +975,11 @@ func buildContainerMountOptions(p types.Project, s types.ServiceConfig, img imag
 					ReadOnly: !m.RW,
 				}
 			}
-			s.Volumes = volumes
+			service.Volumes = volumes
 		}
 	}
 
-	mounts, err := fillBindMounts(p, s, mounts)
+	mounts, err := fillBindMounts(p, service, mounts)
 	if err != nil {
 		return nil, err
 	}
@@ -997,10 +1042,10 @@ func buildContainerConfigMounts(p types.Project, s types.ServiceConfig) ([]mount
 		}
 
 		if definedConfig.Driver != "" {
-			return nil, errors.New("Docker Compose does not support configs.*.driver")
+			return nil, errors.New("Docker Compose does not support configs.*.driver") //nolint:staticcheck
 		}
 		if definedConfig.TemplateDriver != "" {
-			return nil, errors.New("Docker Compose does not support configs.*.template_driver")
+			return nil, errors.New("Docker Compose does not support configs.*.template_driver") //nolint:staticcheck
 		}
 
 		if definedConfig.Environment != "" || definedConfig.Content != "" {
@@ -1047,10 +1092,10 @@ func buildContainerSecretMounts(p types.Project, s types.ServiceConfig) ([]mount
 		}
 
 		if definedSecret.Driver != "" {
-			return nil, errors.New("Docker Compose does not support secrets.*.driver")
+			return nil, errors.New("Docker Compose does not support secrets.*.driver") //nolint:staticcheck
 		}
 		if definedSecret.TemplateDriver != "" {
-			return nil, errors.New("Docker Compose does not support secrets.*.template_driver")
+			return nil, errors.New("Docker Compose does not support secrets.*.template_driver") //nolint:staticcheck
 		}
 
 		if definedSecret.Environment != "" {
@@ -1095,28 +1140,22 @@ func isUnixAbs(p string) bool {
 }
 
 func isWindowsAbs(p string) bool {
-	if strings.HasPrefix(p, "\\\\") {
-		return true
-	}
-	if len(p) > 2 && p[1] == ':' {
-		return p[2] == '\\'
-	}
-	return false
+	return paths.IsWindowsAbs(p)
 }
 
 func buildMount(project types.Project, volume types.ServiceVolumeConfig) (mount.Mount, error) {
 	source := volume.Source
-	// on windows, filepath.IsAbs(source) is false for unix style abs path like /var/run/docker.sock.
-	// do not replace these with  filepath.Abs(source) that will include a default drive.
-	if volume.Type == types.VolumeTypeBind && !filepath.IsAbs(source) && !strings.HasPrefix(source, "/") {
-		// volume source has already been prefixed with workdir if required, by compose-go project loader
-		var err error
-		source, err = filepath.Abs(source)
-		if err != nil {
-			return mount.Mount{}, err
+	switch volume.Type {
+	case types.VolumeTypeBind:
+		if !filepath.IsAbs(source) && !isUnixAbs(source) && !isWindowsAbs(source) {
+			// volume source has already been prefixed with workdir if required, by compose-go project loader
+			var err error
+			source, err = filepath.Abs(source)
+			if err != nil {
+				return mount.Mount{}, err
+			}
 		}
-	}
-	if volume.Type == types.VolumeTypeVolume {
+	case types.VolumeTypeVolume:
 		if volume.Source != "" {
 			pVolume, ok := project.Volumes[volume.Source]
 			if ok {
@@ -1125,7 +1164,7 @@ func buildMount(project types.Project, volume types.ServiceVolumeConfig) (mount.
 		}
 	}
 
-	bind, vol, tmpfs := buildMountOptions(volume)
+	bind, vol, tmpfs, img := buildMountOptions(volume)
 
 	if bind != nil {
 		volume.Type = types.VolumeTypeBind
@@ -1140,37 +1179,35 @@ func buildMount(project types.Project, volume types.ServiceVolumeConfig) (mount.
 		BindOptions:   bind,
 		VolumeOptions: vol,
 		TmpfsOptions:  tmpfs,
+		ImageOptions:  img,
 	}, nil
 }
 
-func buildMountOptions(volume types.ServiceVolumeConfig) (*mount.BindOptions, *mount.VolumeOptions, *mount.TmpfsOptions) {
+func buildMountOptions(volume types.ServiceVolumeConfig) (*mount.BindOptions, *mount.VolumeOptions, *mount.TmpfsOptions, *mount.ImageOptions) {
+	if volume.Type != types.VolumeTypeBind && volume.Bind != nil {
+		logrus.Warnf("mount of type `%s` should not define `bind` option", volume.Type)
+	}
+	if volume.Type != types.VolumeTypeVolume && volume.Volume != nil {
+		logrus.Warnf("mount of type `%s` should not define `volume` option", volume.Type)
+	}
+	if volume.Type != types.VolumeTypeTmpfs && volume.Tmpfs != nil {
+		logrus.Warnf("mount of type `%s` should not define `tmpfs` option", volume.Type)
+	}
+	if volume.Type != types.VolumeTypeImage && volume.Image != nil {
+		logrus.Warnf("mount of type `%s` should not define `image` option", volume.Type)
+	}
+
 	switch volume.Type {
 	case "bind":
-		if volume.Volume != nil {
-			logrus.Warnf("mount of type `bind` should not define `volume` option")
-		}
-		if volume.Tmpfs != nil {
-			logrus.Warnf("mount of type `bind` should not define `tmpfs` option")
-		}
-		return buildBindOption(volume.Bind), nil, nil
+		return buildBindOption(volume.Bind), nil, nil, nil
 	case "volume":
-		if volume.Bind != nil {
-			logrus.Warnf("mount of type `volume` should not define `bind` option")
-		}
-		if volume.Tmpfs != nil {
-			logrus.Warnf("mount of type `volume` should not define `tmpfs` option")
-		}
-		return nil, buildVolumeOptions(volume.Volume), nil
+		return nil, buildVolumeOptions(volume.Volume), nil, nil
 	case "tmpfs":
-		if volume.Bind != nil {
-			logrus.Warnf("mount of type `tmpfs` should not define `bind` option")
-		}
-		if volume.Volume != nil {
-			logrus.Warnf("mount of type `tmpfs` should not define `volume` option")
-		}
-		return nil, nil, buildTmpfsOptions(volume.Tmpfs)
+		return nil, nil, buildTmpfsOptions(volume.Tmpfs), nil
+	case "image":
+		return nil, nil, nil, buildImageOptions(volume.Image)
 	}
-	return nil, nil, nil
+	return nil, nil, nil, nil
 }
 
 func buildBindOption(bind *types.ServiceVolumeBind) *mount.BindOptions {
@@ -1199,7 +1236,7 @@ func buildVolumeOptions(vol *types.ServiceVolumeVolume) *mount.VolumeOptions {
 	return &mount.VolumeOptions{
 		NoCopy:  vol.NoCopy,
 		Subpath: vol.Subpath,
-		// Labels:       , // FIXME missing from model ?
+		Labels:  vol.Labels,
 		// DriverConfig: , // FIXME missing from model ?
 	}
 }
@@ -1214,13 +1251,22 @@ func buildTmpfsOptions(tmpfs *types.ServiceVolumeTmpfs) *mount.TmpfsOptions {
 	}
 }
 
+func buildImageOptions(image *types.ServiceVolumeImage) *mount.ImageOptions {
+	if image == nil {
+		return nil
+	}
+	return &mount.ImageOptions{
+		Subpath: image.SubPath,
+	}
+}
+
 func (s *composeService) ensureNetwork(ctx context.Context, project *types.Project, name string, n *types.NetworkConfig) (string, error) {
 	if n.External {
 		return s.resolveExternalNetwork(ctx, n)
 	}
 
 	id, err := s.resolveOrCreateNetwork(ctx, project, name, n)
-	if errdefs.IsConflict(err) {
+	if cerrdefs.IsConflict(err) {
 		// Maybe another execution of `docker compose up|run` created same network
 		// let's retry once
 		return s.resolveOrCreateNetwork(ctx, project, name, n)
@@ -1229,6 +1275,9 @@ func (s *composeService) ensureNetwork(ctx context.Context, project *types.Proje
 }
 
 func (s *composeService) resolveOrCreateNetwork(ctx context.Context, project *types.Project, name string, n *types.NetworkConfig) (string, error) { //nolint:gocyclo
+	// This is containers that could be left after a diverged network was removed
+	var dangledContainers Containers
+
 	// First, try to find a unique network matching by name or ID
 	inspect, err := s.apiClient().NetworkInspect(ctx, n.Name, network.InspectOptions{})
 	if err == nil {
@@ -1262,7 +1311,7 @@ func (s *composeService) resolveOrCreateNetwork(ctx context.Context, project *ty
 				return inspect.ID, nil
 			}
 
-			err = s.removeDivergedNetwork(ctx, project, name, n)
+			dangledContainers, err = s.removeDivergedNetwork(ctx, project, name, n)
 			if err != nil {
 				return "", err
 			}
@@ -1279,8 +1328,8 @@ func (s *composeService) resolveOrCreateNetwork(ctx context.Context, project *ty
 	}
 
 	// NetworkList Matches all or part of a network name, so we have to filter for a strict match
-	networks = utils.Filter(networks, func(net network.Summary) bool {
-		return net.Name == n.Name
+	networks = slices.DeleteFunc(networks, func(net network.Summary) bool {
+		return net.Name != n.Name
 	})
 
 	for _, net := range networks {
@@ -1359,10 +1408,16 @@ func (s *composeService) resolveOrCreateNetwork(ctx context.Context, project *ty
 		return "", fmt.Errorf("failed to create network %s: %w", n.Name, err)
 	}
 	w.Event(progress.CreatedEvent(networkEventName))
+
+	err = s.connectNetwork(ctx, n.Name, dangledContainers, nil)
+	if err != nil {
+		return "", err
+	}
+
 	return resp.ID, nil
 }
 
-func (s *composeService) removeDivergedNetwork(ctx context.Context, project *types.Project, name string, n *types.NetworkConfig) error {
+func (s *composeService) removeDivergedNetwork(ctx context.Context, project *types.Project, name string, n *types.NetworkConfig) (Containers, error) {
 	// Remove services attached to this network to force recreation
 	var services []string
 	for _, service := range project.Services.Filter(func(config types.ServiceConfig) bool {
@@ -1379,13 +1434,54 @@ func (s *composeService) removeDivergedNetwork(ctx context.Context, project *typ
 		Project:  project,
 	})
 	if err != nil {
-		return err
+		return nil, err
+	}
+
+	containers, err := s.getContainers(ctx, project.Name, oneOffExclude, true, services...)
+	if err != nil {
+		return nil, err
+	}
+
+	err = s.disconnectNetwork(ctx, n.Name, containers)
+	if err != nil {
+		return nil, err
 	}
 
 	err = s.apiClient().NetworkRemove(ctx, n.Name)
 	eventName := fmt.Sprintf("Network %s", n.Name)
 	progress.ContextWriter(ctx).Event(progress.RemovedEvent(eventName))
-	return err
+	return containers, err
+}
+
+func (s *composeService) disconnectNetwork(
+	ctx context.Context,
+	network string,
+	containers Containers,
+) error {
+	for _, c := range containers {
+		err := s.apiClient().NetworkDisconnect(ctx, network, c.ID, true)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+func (s *composeService) connectNetwork(
+	ctx context.Context,
+	network string,
+	containers Containers,
+	config *network.EndpointSettings,
+) error {
+	for _, c := range containers {
+		err := s.apiClient().NetworkConnect(ctx, network, c.ID, config)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
 }
 
 func (s *composeService) resolveExternalNetwork(ctx context.Context, n *types.NetworkConfig) (string, error) {
@@ -1403,18 +1499,19 @@ func (s *composeService) resolveExternalNetwork(ctx context.Context, n *types.Ne
 	if len(networks) == 0 {
 		// in this instance, n.Name is really an ID
 		sn, err := s.apiClient().NetworkInspect(ctx, n.Name, network.InspectOptions{})
-		if err != nil && !errdefs.IsNotFound(err) {
+		if err == nil {
+			networks = append(networks, sn)
+		} else if !cerrdefs.IsNotFound(err) {
 			return "", err
 		}
-		networks = append(networks, sn)
+
 	}
 
 	// NetworkList API doesn't return the exact name match, so we can retrieve more than one network with a request
-	networks = utils.Filter(networks, func(net network.Inspect) bool {
-		// later in this function, the name is changed the to ID.
+	networks = slices.DeleteFunc(networks, func(net network.Inspect) bool {
 		// this function is called during the rebuild stage of `compose watch`.
 		// we still require just one network back, but we need to run the search on the ID
-		return net.Name == n.Name || net.ID == n.Name
+		return net.Name != n.Name && net.ID != n.Name
 	})
 
 	switch len(networks) {
@@ -1441,7 +1538,7 @@ func (s *composeService) resolveExternalNetwork(ctx context.Context, n *types.Ne
 func (s *composeService) ensureVolume(ctx context.Context, name string, volume types.VolumeConfig, project *types.Project, assumeYes bool) (string, error) {
 	inspected, err := s.apiClient().VolumeInspect(ctx, volume.Name)
 	if err != nil {
-		if !errdefs.IsNotFound(err) {
+		if !cerrdefs.IsNotFound(err) {
 			return "", err
 		}
 		if volume.External {
